socket = require 'socket'

local host = "www.w3.org"
local threads = {} -- declare the table in which to put ids of coroutines that yield

function download (host, file)
  local connection = assert (socket.connect(host, 80))
  local count = 0
  connection:send("GET " .. file .. " HTTP/1.0\r\n\r\n")
  while true do
    local data, status = receive(connection)
    count = count + #data
    if status == "closed" then break end
  end
  connection:close()
  print (file, count)
end

function receive (connection)
  connection:settimeout(0)
  local data, status, partialData = connection:receive(2^10)
  if status == "timeout" then
    coroutine.yield(connection)
  end
  return data or partialData, status
end

function get (host, file)
  local co = coroutine.create(
    function ()
      download(host, file)
    end)

  table.insert(threads,co)
end

function dispatch()
  local i = 1
  local timedout = {}  -- table will hold threads that timed out so they can be restared
  while true do
    if threads[i] == nil then  -- at end of threads list?
      if threads[1] == nil then break end  -- are all threads finished?
      i = 1   -- restart at first thread
      timedout = {}  -- reset the timedout table to go through them again
    end
    local status, res = coroutine.resume(threads[i])
    if not res then  -- thread finished its task?
      table.remove(threads,i) -- remove from the table and change table size
    else
      i = i +1
      timedout[#timedout + 1] = res -- thread timedout so put it in timedout table
      if #timedout == #threads then  -- are all threads blocked?
        socket.select(timedout) -- if so then wait until the state of one of them changes
      end
    end
  end
end

-- now start up several coroutines to get pages over the web

get(host, "/Consortium/")
get(host, "/participate/")
get(host, "/")
get(host, "/Consortium/membership")
dispatch() -- start the main loop in the dispatcher



